package modespahe.test;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.UUID;
import javax.jcr.Binary;
import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.Session;
import org.modeshape.jcr.JcrConfiguration;
import org.modeshape.jcr.JcrEngine;
import org.modeshape.jcr.JcrTools;
import org.modeshape.sequencer.classfile.ClassFileSequencerLexicon;
import org.modeshape.connector.disk.DiskSource;
import javax.annotation.PostConstruct;
import javax.ejb.Local;
import javax.ejb.Singleton;
import javax.ejb.Startup;

/**
 * Session Bean implementation class EjbTest
 */
@Singleton
@Startup
@Local
public class EjbTest implements EjbTestI {

    private String repositoryName = "content";
    private String workspaceName = "default";
    private JcrConfiguration configuration;
    private JcrEngine engine;
    private Repository repository;
    private static ClassLoader classLoader;

    /**
     * Default constructor.
     */
    public EjbTest() {
        classLoader = Thread.currentThread().getContextClassLoader();
        configuration = new JcrConfiguration();
    }

    @SuppressWarnings("unused")
    @PostConstruct
    private void initialize() {

        configuration
                .repositorySource("store")
                .usingClass(DiskSource.class)
                .setDescription("The repository for our content")
                .setProperty("repositoryRootPath", "/content/someApp")
                .setProperty("defaultWorkspaceName", workspaceName)
                .setProperty("predefinedWorkspaceNames",
                        new String[] { "default", "dev" })
                .setProperty("rootNodeUuid",
                        UUID.fromString("fd129c12-81a8-42ed-aa4b-820dba49e6f0"))
                .setProperty("updatesAllowed", "true")
                .setProperty("creatingWorkspaceAllowed", "false");

        String sequencingPath = getClass().getClassLoader()
                .getResource("/sequencing.cnd").getPath();

        // the following transformation is made because of weird path returned
        // by the getResource method
        sequencingPath ="c:\\sequencing.cnd";
//        sequencingPath = sequencingPath
//                .substring(
//                        0,
//                        sequencingPath
//                                .indexOf("bin/content/testEJB-2.8.2.Final.war/WEB-INF/classes/sequencing.cnd"));
//        sequencingPath = sequencingPath
//                + "/standalone/deployments/testEJB-2.8.2.Final.war/WEB-INF/classes/sequencing.cnd";
        System.out.println("######" + sequencingPath);
        // Set up the JCR repository to use the source ...
        configuration
                .repository(repositoryName)
                .addNodeTypes(sequencingPath)
                .registerNamespace(ClassFileSequencerLexicon.Namespace.PREFIX,
                        ClassFileSequencerLexicon.Namespace.URI)
                .setSource("store");

        configuration
                .sequencer("ZIP Sequencer")
                .usingClass("org.modeshape.sequencer.zip.ZipSequencer")
                .loadedFromClasspath()
                .setDescription(
                        "Sequences compressed files to extract the internal file and folder structure")
                .sequencingFrom(
                        "//(*.(zip|gz|jar|war|ear)[*])/jcr:content[@jcr:data]")
                .andOutputtingTo("/zip/$1");

        // Now start the client and tell it which repository and workspace to
        // use ...
        if (this.repository == null) {
            try {
                // Start the ModeShape engine ...
                this.engine = this.configuration.build();
                this.engine.start();

                // Now get the repository instance ...
                this.repository = this.engine.getRepository(repositoryName);

            } catch (Exception e) {
                this.repository = null;
                System.err.println(e.getMessage());
            }
        }

        try {

           // URL testUrl = getClass().getClassLoader().getResource("/test.zip");
            String testPath ="c:\\test.zip";
//            String testPath = testUrl
//                    .getPath()
//                    .substring(
//                            0,
//                            sequencingPath
//                                    .indexOf("bin/content/testEJB-2.8.2.Final.war/WEB-INF/classes/test.zip"));
//            testPath = testPath
//                    + "/standalone/deployments/testEJB-2.8.2.Final.war/WEB-INF/classes/test.zip";
            System.out.println("######" + testPath);
            // upload a zip file
            uploadFile(new URL("file:" + testPath));
        } catch (MalformedURLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void uploadFile(URL url) throws Exception {

        // Grab the last segment of the URL path, using it as the filename
        String filename = url.getPath().replaceAll("([^/]*/)*", "");
        String nodePath = "/a/b/" + filename;
        String mimeType = "application/zip";

        // Now use the JCR API to upload the file ...
        Session session = this.repository.login();
        JcrTools tools = new JcrTools();
        try {

            // Create the node at the supplied path ...
            Node node = tools.findOrCreateNode(session, nodePath, "nt:folder",
                    "nt:file");

            // Upload the file to that node ...
            Node contentNode = tools.findOrCreateChild(node, "jcr:content",
                    "nt:resource");
            contentNode.setProperty("jcr:mimeType", mimeType);
            contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
            Binary binaryValue = session.getValueFactory().createBinary(
                    url.openStream());
            contentNode.setProperty("jcr:data", binaryValue);

            // Save the session ...
            session.save();

            waitUntilSequencedNodesIs(1);
        } finally {
            session.logout();
        }
    }

    private void waitUntilSequencedNodesIs(int totalNumberOfNodesSequenced)

    throws InterruptedException {
        // check 50 times, waiting 0.1 seconds between (for a total of 5 seconds
        // max) ...
        long numFound = 0;

        for (int i = 0; i != 50; i++) {

            numFound = engine.getSequencingService().getStatistics()
                    .getNumberOfNodesSequenced();

            System.out
                    .println("Number of nodes to be sequenced is " + numFound);

            if (numFound >= totalNumberOfNodesSequenced) {
                // Wait for the sequenced output to be saved before searching
                // ...
                Thread.sleep(500);
                return;
            }
        }
    }
}
